package com.acme;

import java.io.IOException;
import java.util.concurrent.RejectedExecutionException;
import java.util.concurrent.SynchronousQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicLong;
import org.glassfish.grizzly.CompletionHandler;
import org.glassfish.grizzly.http.server.HttpHandler;
import org.glassfish.grizzly.http.server.Request;
import org.glassfish.grizzly.http.server.Response;


public class MyHandler extends HttpHandler
{
    private static final int MAX_REQUEST = 500;
    
    private ThreadPoolExecutor          executor;
    private CompletionHandler<Response> completion;
    private AtomicLong                  adapterChooser;
    private AtomicLong                  rejectedChooser;
    private HttpHandler                 target;
    

    public MyHandler()
        throws IOException
    {
        executor = new ThreadPoolExecutor( 0, MAX_REQUEST, 5*60, TimeUnit.SECONDS, new SynchronousQueue<Runnable>() );
        completion = new MyCompletionHandler();
        adapterChooser = new AtomicLong( -1 );
        rejectedChooser = new AtomicLong( -1 );
        target = new MyComplexHandler( false );
    }

    @Override
    public void service( Request req, Response res )
        throws Exception
    {
        res.suspend( Long.MAX_VALUE, TimeUnit.MINUTES, completion );
        try
        {
            executor.execute( new MyExecutor(req,res,target) );
        }
        catch( RejectedExecutionException ex )
        {
            res.setStatus( 509, "Too many request" );
            res.resume();
        }
    }


//    private Adapter chooseAdapter()
//    {
//        if( (adapterChooser.incrementAndGet() % 2) == 0)
//        {
//            return new MySimpleAdapter( (rejectedChooser.incrementAndGet() % 5) == 0 );
//        }
//        else
//        {
//            return new MyGrizzlyAdapter( (rejectedChooser.incrementAndGet() % 5) == 0 );
//        }
//    }

}
