/*
 * Copyright (c) 2011 by Axway Software
 * All brand or product names are trademarks or registered trademarks
 * of their respective holders.
 * This document and the software described in this document are the property
 * of Axway Software and are protected as Axway Software trade secrets.
 * No part of this work may be reproduced or disseminated in any form or
 * by any means, without the prior written permission of Axway Software.
 */

package com.acme.upload;

import com.acme.NullOutputStream;
import java.io.OutputStream;
import java.util.concurrent.atomic.AtomicInteger;
import org.glassfish.grizzly.http.server.Request;
import org.glassfish.grizzly.http.server.Response;

/**
 *
 * @author dgay
 */
public class Upload
{
    private static final int MAX_BUFFER_SIZE = 64 * 1024;
    private static final AtomicInteger COUNT = new AtomicInteger();

    private int             id;
    private Request         req;
    private Response        res;
    private OutputStream    os;
    private byte[]          buffer;


    public Upload( Request req, Response res )
    {
        this.id = COUNT.incrementAndGet();
        this.req = req;
        this.res = res;
        this.os = new NullOutputStream();
        this.buffer = new byte[MAX_BUFFER_SIZE];
    }

    @Override
    public String toString()
    {
        return "Upload "+id;
    }

    public boolean upload()
    {
        boolean finish;
        int     read;

        finish = false;
        try
        {
            read = req.getInputStream( false ).read( buffer );
            //read = req.getReader().read( buffer );
            //System.out.println( Thread.currentThread().getName()+" is reading : "+read+" bytes " );
            if( read!=-1 )
            {
                os.write( buffer, 0, read );
            }
            else
            {
                res.setStatus( 200, "Ok, Done upload" );
                finish = true;
                resumeResponse();
            }
        }
        catch( Exception ex )
        {
            ex.printStackTrace( System.err );
            res.setStatus( 500, "Error while processing the upload !" );
            res.setHeader( "Connection", "close" );
            resumeResponse();
        }
        finally
        {
            return finish;
        }
    }

	private void resumeResponse()
	{
		try
		{
			os.close();
		}
		catch ( Exception ex )
		{
			ex.printStackTrace( System.err );
		}

        try
        {
            res.resume();
        }
        catch( Exception ex )
        {
            ex.printStackTrace( System.err );
        }
	}

}
