package com.acme.test;

import org.apache.commons.httpclient.MultiThreadedHttpConnectionManager;
import com.acme.MyHandler;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicInteger;
import org.apache.commons.httpclient.HttpConnectionManager;
import org.glassfish.grizzly.http.server.HttpHandler;
import org.glassfish.grizzly.http.server.HttpServer;
import org.glassfish.grizzly.http.server.ServerConfiguration;
import org.glassfish.grizzly.http.server.NetworkListener;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import static org.junit.Assert.*;

/**
 *
 */
public class TestSuspend
{
    private static final int NB_CLIENT = 100;
    private static final int NB_PASS = 23;
    
    int                     port = 35000;
    HttpServer              server;
    HttpHandler             handler;
    String                  url;
    AtomicInteger           succes;
    AtomicInteger           busy;
    HttpConnectionManager   manager;

    static
    {
        //Logger.getLogger( "org.apache.commons.httpclient" ).setLevel( Level.OFF );
    }

    @Before
    public void before()
        throws Exception
    {
        NetworkListener nl;
        // Create the adapter
        handler = new MyHandler( );

        // Create the Server
        server = HttpServer.createSimpleServer( ".", port );
        removeDefaultBehavior( server );
        server.getServerConfiguration().addHttpHandler( handler, "/" );
        
        nl = server.getListeners().iterator().next();
        nl.setRcmSupportEnabled( false );
        nl.setCompression( "on" );
        nl.setChunkingEnabled( true );
        nl.setDisableUploadTimeout( true );
        
        // Start
        server.start();

        // Others
        succes = new AtomicInteger(0);
        busy = new AtomicInteger(0);
        url = "http://localhost:"+port+"/ping";
        manager = new MultiThreadedHttpConnectionManager();
        manager.getParams().setDefaultMaxConnectionsPerHost( NB_CLIENT*2 );
        manager.getParams().setMaxTotalConnections( NB_CLIENT*2 );
        
    }

    @After
    public void after()
    {
        server.stop();
    }

    
    @Test
    public void testManyRequestsInParallel()
        throws Exception
    {
        ThreadPoolExecutor  executor;

        executor = new ThreadPoolExecutor( NB_CLIENT, NB_CLIENT, 5*60, TimeUnit.SECONDS, new LinkedBlockingQueue<Runnable>() );

        // Start the clients
        for( int i=0; i<NB_CLIENT; i++ )
        {
            executor.execute( new Caller(url,NB_PASS,succes,busy,manager) );
        }

        // Wait for the end
        try
		{
			executor.shutdown();
			executor.awaitTermination( 15, TimeUnit.MINUTES );
		}
		catch (InterruptedException e)
		{
			fail( "Interrupted during requesters execution." );
		}

        // Show results
        System.out.println();
        System.out.println( "** Success: "+succes.get() );
        System.out.println( "** Busy   : "+busy.get() );
        System.out.println( "** Total  : "+(3*NB_CLIENT*NB_PASS) );
        System.out.println();

        assertEquals( (3*NB_CLIENT*NB_PASS), succes.get()+busy.get() );
    }

    private void removeDefaultBehavior( HttpServer server )
    {
        ServerConfiguration conf;

        conf = server.getServerConfiguration();

        // Remove all default handler added by default
        for( HttpHandler h : conf.getHttpHandlers().keySet().toArray(new HttpHandler[0]) )
        {
            conf.removeHttpHandler( h );
        }

    }

}
